<?php
/*
 * Belong is some basic sample PHP code used to authenticate against the department CAS server
 * and optionally authorize that the authenticated user is a member of a specific LDAP group.
 *
 * The intention of this code is to demonstrate how to use the phpCAS library to authenticate,
 * and how to use the PHP ldap functions to query our LDAP server for attributes.
 * A decent understanding of PHP is required.
 */

// Authorized LDAP groups. User must be part of one of these groups to be allowed access.
// To disable LDAP authorization for this sample code, just remove all entries from this array.
$authorized_groups = array(
  'csstaff',
  'fac',
  'office',
  'grad',
  'ugrad',
  'research',
  'guest',
);

session_start();

// import phpCAS lib
include_once('./CAS-1.3.3/CAS.php');

// Uncomment to enable debugging for CAS. The web server user needs write permission to this file.
// phpCAS::setDebug('cas_debug.log');

// initialize phpCAS
phpCAS::client(CAS_VERSION_2_0, 'signon.cs.princeton.edu', 443, '/');

// IMPORTANT!!! In production you should tell CAS to validate the authenticity of the TLS
// certificate on our CAS server. To do that, you need to pass in a chain of trusted CA
// certificates.
//phpCAS::setCasServerCACert('/path/to/ca.pem');
phpCAS::setNoCasServerValidation();

// Force the user to authenticate with the CAS server.
// The user will be returned to this page after successfully authenticating.
phpCAS::forceAuthentication();

// At this point the user is authenticated.
$username = phpCAS::getUser();

// Checks to see if we should log the user out. 
if (isset($_REQUEST['logout'])) {
  phpCAS::logout();
}

function check_authorization($authorized_groups, $username) {
  if (empty($authorized_groups)) {
    return TRUE;
  }

  // Check this users LDAP group for authorization
  if (!empty($authorized_groups)) {
    $ldap_connection = ldap_connect('ldap://ldap.cs.princeton.edu');

    // Query user information from LDAP to get their group ID
    $user_base = 'ou=People,dc=cs,dc=princeton,dc=edu';
    $user_search = 'uid=' . 'bosborne';

    $search_result = ldap_search($ldap_connection, $user_base, $user_search);
    $rec = ldap_get_entries($ldap_connection, $search_result);

    // Make sure user exists and has a group ID
    if ($rec['count'] > 0 && isset($rec[0]['gidnumber']) && $rec[0]['gidnumber']['count'] > 0) {
      $gid = $rec[0]['gidnumber'][0];

      // Query group information based on group ID to get the group name.
      $group_base = 'ou=Group,dc=cs,dc=princeton,dc=edu';
      $group_search = 'gidnumber=' . $gid;
      $search_result = ldap_search($ldap_connection, $group_base, $group_search);
      $rec = ldap_get_entries($ldap_connection, $search_result);

      if ($rec['count'] > 0 && isset($rec[0]['cn']) && $rec[0]['gidnumber']['count'] > 0) {
        $groupname = $rec[0]['cn'][0];

        if (in_array($groupname, $authorized_groups)) {
          return TRUE;
        }
      }
    }
  }

  return FALSE;
}

$authorized = check_authorization($authorized_groups, $username);
?>
<!doctype html>
<html>
  <head>
    <title>Sample Application</title>
  </head>
  <body>
    <?php if ($authorized) { ?>
      <h1>Welcome</h1>
      <p>
        You are logged in as <?php echo $username; ?>.
      </p>
    <?php } else { ?>
      <h1>Not Authorized</h1>
      <p>
        You logged in as <?php echo $username; ?> but you are not authorized to view this page.
      </p>
    <?php } ?>
    <p>
      <a href="?logout=">Logout of CAS</a>
    </p>
  </body>
</html>
